/* TiVoCrack
**
**  Brute force decode a SHA1 encoded value.  Used to determine
**  the back door code on a TiVo.
**
**  This version is multithreaded to take advantage of my
**  multi-proc box.
**
**  -- Scott Seligman <scott@scottandmichelle.net>
**
**  Changes:
**   2002-10-28 - Initial version (1.0)
**   2002-10-29 - Added Ctrl-C handler
**                Memory improvements
**                Optional timing
**                Using the internet (1.1)
**   2002-10-30 - A few changes internally to allow the alphabet
**                 and key to change
**                Moved over to another server (1.2)
**                Fixed a bug that skipped the first word in 
**                 a set (1.3)
**                Fixed the local mode
**                Added an option to change the sleep intervals
**                Added an option to lower the priority of the 
**                 worker thread (1.4)
**   2002-10-31 - Changed the default username to anonymous
**                Retries failures on Internet comm
**                Some cleanup of the output
**                Fixed win98
**                Report version to server (1.5)
**                Option to control output (1.6)
**                Added support for windowless build
**   2002-11-01 - PocketPC support added, you're on your own for 
**                 the compile
**                Support for proofs to be sent
**   2002-11-02 - Added xtrack* stuff to check for mem leaks
**                Server protocol v2
**   2002-11-04 - Support for switches with / or -
**                Added an infinte retry option as default
**                Removed the date from the timestamp
**                Fixed pattern bug when pattern included non
**                 alphabet characters
**   2002-11-05 - LowPri now lowers process level too (from 1.61)
**                Better proxy handling (from 1.61)
**                Massive changes to support basic windows
**                Pause functionality
**                Service support (2.00)
**   2002-11-08 - Added ASM version of SHA code
**                Added option to compute kkeys/sec (2.01)
**                Backed out the ASM code for thread issues (2.02)
**   2002-11-09 - Removed the cache system for some speed 
**                 improvements (2.03)
**                Added verify on close and ability to abort after
**                 after the current work unit (2.04)
**   2002-11-10 - Fix bug with endian reversing of the proof (2.05)
**
*/

#ifndef TIVOCRACK_H__INCLUDED__
#define TIVOCRACK_H__INCLUDED__

#include "main.h"
#include "sha1.h"

// ------------------- Prototypes -------------------

// Struct for hash to be stored in
  // Process the command line
bool ProcessCommandLine();

  // Find the next word based off of the pattern
bool FindNextWord(char * szAlphabet, int cAlphabet, char * szPattern, 
                int cPattern, int * nCurrentWord, char * szCurrentWord);

  // Dump the settings to the user
void DumpSettings();

  // The main worker thread
DWORD WINAPI FindTiVoKey(LPVOID);

  // The watcher thread
DWORD WINAPI WatcherThread(LPVOID);

  // Helper function to write a string to the console and a log
void WriteInfo(char * szOut);

  // Determines if a unhashed string matches the hash from TiVo
int IsTiVoKey(DWORD * pdwProof, SHA1_CONTEXT * sha1, 
              char * szString, int nLen);

  // Gracefully handle Ctrl-C
BOOL WINAPI CtrlHandler(DWORD);

  // Encode a CGI query
void SafeAdd(char* szURL, char * szBefore, char* szAdd);

  // Get the next work load from the server
bool GetNextWorkLoad();
bool PutNextWorkLoad();
typedef bool (*WorkLoadFunc)();
bool SafeNextWorkLoad(WorkLoadFunc pFunc); // Wrapper for Get/Put

  // Set the starting point
void SetStart();

  // Initialization
void TiVoCrack_Init();
void TiVoCrack_Uninit();

  // Some helper functions to track the threads
void AddThread();
void RemoveThread();

// ------------------- Globals ----------------------
extern char    g_szCurrentWord[MAX_LETTERS+1]; // The next word to try
extern int     g_nCurrentWord[MAX_LETTERS+1]; // Pos in g_szAlphabet
extern int     g_cCurrentWord; // Number of letters in word
extern bool    g_bWorkerRunning; // Worker threads should be running?
extern int     g_nWorkerRunning; // Number of running worker threads
extern char    g_szMessage[MAX_STRING]; // Message to pass from worker
extern bool    g_bNoMore; // Shutting down, no more work
extern bool    g_bShutdownWindow; // After a shutdown, close the main window?
extern bool    g_bFinishCurrent; // Abort after this work unit?
extern bool    g_bNoMoreWords; // Any more words?
extern HANDLE  g_eventStop; // Event to trigger the UI thread to stop sleeping
extern CRITICAL_SECTION g_crit; // To protect everything
extern CThread *    g_pThread; // Handle to linked list of threads
extern DWORD   g_dwTickStart; // To computer the number of kkeys/sec
extern DWORD   g_dwTickEnd;
extern double  g_nKeysCalced;

// The hash itself.  As an array of DWORDs for simplicity's sake.
// g_dwKey2 is just the endian reversed version of g_dwKey2
// -- Version 3.0 == "3 0 BC"
//5CA5D9DBE5338BAB8690C79C9A9310BCD3A8F23B
// -- Version 3.2 == ????????
//96F8B204FD99534759A6C11A181EEDDFEB2DF1D4
extern DWORD   g_dwKey1[5];
extern DWORD   g_dwKey2[5];

extern CSettings *  g_pSettings;
extern CWorkUnit *  g_pWorkUnit;
extern CWorkUnit ** g_pWorkUnits;
extern int          g_cWorkUnits;


#ifdef TIME_CALCULATION
#define TIME_COUNT 5
extern DWORD   g_dwFindWordStart[TIME_COUNT];
extern DWORD   g_dwFindWordEnd[TIME_COUNT];
extern DWORD   g_dwBruteStart[TIME_COUNT];
extern DWORD   g_dwBruteEnd[TIME_COUNT];
extern DWORD   g_dwBrute;
extern DWORD   g_dwFind;
extern int     g_nCurTime;
#endif

#endif //TIVOCRACK_H__INCLUDED__
