#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include <openssl/sha.h>
//#include "/usr/include/openssl/sha.h"

#include "Cracker.h"
extern "C" {
char * _sha1(char * msg, unsigned long len, char * dgst);
}
#define HASH_LEN 20

Cracker::Cracker()
{
  _alphabet=NULL;
  _solution=NULL;

  _statusEvery=4;

  _ctext0123=(char*) calloc(HASH_LEN,1);
  _ctext3210=(char*) calloc(HASH_LEN,1);
  _ctext1032=(char*) calloc(HASH_LEN,1);
  _ctext2301=(char*) calloc(HASH_LEN,1);

  _md=(char*) calloc(320,1);

  _testCount=0;
  _proof=0;
  _showProgress=true;
  _statusEvery=4;
}

Cracker::~Cracker()
{
  if (_alphabet!=NULL)
    free(_alphabet);
  if (_solution!=NULL)
    free(_solution);
  free(_ctext0123);
  free(_ctext3210);
  free(_ctext1032);
  free(_ctext2301);
}

void Cracker::enableProgress(bool enable)
{
  _showProgress=enable;
}

unsigned int Cracker::getProof()
{
  return _proof;
}

void Cracker::setCtext(char *ctext)
{
  // convert the ascii into the key.
  for (int i=0;i<HASH_LEN;i++)
    {
      _ctext0123[i]=(hexValue(ctext[2*i])<<4)
	+hexValue(ctext[2*i+1]);
    }

  // compute endian-reversed version, assuming 32b
  // words.
  for (int i=0;i<HASH_LEN;i++)
    {
      _ctext3210[i]=_ctext0123[3-(i%4)+((i>>2)<<2)];
      _ctext1032[i]=_ctext0123[1-(i%2)+((i>>1)<<1)];
    }

  /** 2301 uses 3210, so we have to compute this in a separate step. */
  for (int i=0;i<HASH_LEN;i++)
    {
      _ctext2301[i]=_ctext3210[1-(i%2)+((i>>1)<<1)];
    }
}

void Cracker::setAlphabet(char *alphabet)
{
  if (_alphabet!=NULL)
    free(_alphabet);

  _alphabet=strdup(alphabet);
  _alphabetLen=strlen(_alphabet);
}

void Cracker::setProgressFrequency(int chars)
{
  _statusEvery=chars;
}

bool Cracker::searchpattern(char *pattern_in, char *seed)
{
  char *pattern,*state;
  int patternlen;

  //pattern=strdup(pattern_in);
  
  patternlen=strlen(pattern_in);
  pattern=(char *)malloc(patternlen+72);
  strcpy(pattern, pattern_in);

  /* modify the pattern to use the seed */
  int seedpos=0, patternpos=0;
  while (pattern[patternpos]!=0 && seed[seedpos]!=0)
    {
      if (pattern[patternpos]=='?')
	pattern[patternpos]=seed[seedpos++];
      patternpos++;
    }

  _statusLen=-1;

  int qcount=0;

  for (int i=patternlen;i>=0;i--)
    if (pattern[i]=='?')
      {
	qcount++;
	if (qcount>_statusEvery)
	  {
	    _statusLen=i;
	    break;
	  }
      }

  //  printf("_statusEvery=%i\t_statusLen=%i\n",_statusEvery,_statusLen);

  state=strdup(pattern);
  _testCount=0;
  _proof=0;

  bool res=searchpattern_nonrec(pattern);

  _proof=htonl(_proof);

  free(pattern);
  free(state);

  return res;
}

bool Cracker::searchpattern_nonrec(char *pattern)
{
  int len=strlen(pattern);
  int numoffsets=0;
  // index into pattern of the nth question mark.
  int *offset=(int*) calloc(sizeof(int),len);
  // position in the alphabet of the nth question mark
  int *state=(int*) calloc(sizeof(int),len);

  // make a table telling us where question marks are.
  // simultaneously, initialize the pattern to the
  // first state.
  for (int i=0;i<len;i++)
    {
      if (pattern[i]=='?')
	{
	  offset[numoffsets]=i;
	  numoffsets++;
	  pattern[i]=_alphabet[0];
	  state[i]=0;
	}
    }

  int pos=0;
  int alphabetlen=strlen(_alphabet);

  // This is essentially a counter in base alphabetlen.
  // mainloop is for each iteration. loop is where we handle
  // carries. 'state' is our number. 'offsets' tells us which
  // characters in pattern correspond to the digits in state.
  // every time we modify a state digit, we update the corresponding
  // character in pattern.
 mainloop:
  pos=numoffsets-1;

  if (testPtext(pattern,len))
    {
      setSolution(pattern);
      return true;
    }

 loop:

  state[pos]++;
  pattern[offset[pos]]=_alphabet[state[pos]];

  // carry?
  if (state[pos]==alphabetlen)
    {
      state[pos]=0;
      pattern[offset[pos]]=_alphabet[0];
      pos--;
      if (pos<0)
	goto done;
      goto loop;
    }

  //  printf("%s\t%i\n",pattern,pos);

  if (pos<(numoffsets-_statusEvery) && _showProgress)
    {
      printf("%s",pattern);
      fflush(NULL);
      // a line feed to bring the cursor back to the left
      printf("%c",13);
    }

  goto mainloop;

 done:

  free(offset);
  free(state);

  return false;
}

// convert ascii letter (hex) into integer.
int Cracker::hexValue(int c)
{
  if (c>='A')
    return c-'A'+10;
  return c-'0';
}

void Cracker::printKey(char *k)
{
  for (int i=0;i<HASH_LEN;i++)
    {
      printf("%02X ",k[i]&0xff);
    }
  printf("\n");
}

bool Cracker::testPtext(char *ptext, int patternlen)
{
  /*  SHA1Context sha1;
      
  SHA1Init(&sha1);
  SHA1Update(&sha1, ptext,patternlen);
  SHA1Final(&sha1,(unsigned char*) md);
  */
  SHA1((unsigned char*) ptext,patternlen, (unsigned char*) _md);
  //  _sha1(ptext, patternlen, md);

  _testCount++;
  _proof=_proof^(((int*)_md)[0]);

  if (testCtext(_md))
    {
      return true;
    }

  return false;
}

bool Cracker::fastMemCmp20(char *p1, char *p2)
{
  long *l1=(long*) p1;
  long *l2=(long*) p2;
  
  if (l1[0]!=l2[0])
    return false;

  return !memcmp(p1,p2,20);
}

bool Cracker::testCtext(char *ctext)
{
  if (fastMemCmp20(ctext,_ctext0123))
    return true;

  if (fastMemCmp20(ctext,_ctext3210))
    return true;

  return false;

  //  if (!memcmp(ctext,_ctext0123,HASH_LEN))
  //    return true;

  //if (!memcmp(ctext,_ctext3210,HASH_LEN))
  //    return true;

  /*
    if (ENDIAN1032)
    {
    if (!memcmp(ctext,CTEXT1032,SHA_DIGEST_LENGTH))
    return 1;
    }
    
    if (ENDIAN2301)
    {
    if (!memcmp(ctext,CTEXT2301,SHA_DIGEST_LENGTH))
    return 1;
    }
  */

  /*
    if (CHECK_TWIDDLE)
    {
    // see if the ciphertext has been xor'd by some 
    // 32 bit constant. endianness is irrelevant.
    long *l1=(long*) ctext;
    long *l2=(long*) CTEXT0123;
    
    if ((l1[0]^l2[0])==(l1[2]^l2[2]) && (l1[0]^l2[0])==(l1[4]^l2[4]))
    {
    printf("\nTwiddle %08lX\n",l1[0]^l2[0]);
    return 1;
    }
    }
  */

  return false;
}

char *Cracker::getSolution()
{
  return _solution;
}

void Cracker::setSolution(char *sol)
{
  if (_solution!=NULL)
    free(_solution);

  _solution=strdup(sol);
}

long Cracker::getCount()
{
  return _testCount;
}
