#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "Table.h"

Table::Table()
{
  _allocated=8;
  _size=0;

  _entries=NULL;
  grow();
}

void Table::grow()
{
  _allocated=_allocated*2;
  if (_allocated<8)
    _allocated=8;

  _entries=(struct TableEntry*) realloc(_entries,sizeof(TableEntry)*_allocated);
}

Table::~Table()
{
  for (int i=0;i<_size;i++)
    {
      free(_entries[i].key);
      free(_entries[i].value);
    }

  free(_entries);
}

int Table::getIndex(const char *key)
{
  for (int i=0;i<_size;i++)
    {
      if (!strcmp(key,_entries[i].key))
	return i;
    }
  return -1;
}

char *Table::get(const char *key)
{
  int i=getIndex(key);

  if (i<0)
    return NULL;

  return(_entries[i].value);
}

char *Table::getKey(int idx)
{
  return (_entries[idx].key);
}

char *Table::getValue(int idx)
{
  return (_entries[idx].value);
}

void Table::put(const char *key, const char *value)
{
  int idx=getIndex(key);

  if (idx==-1)
    {
      if ((_size+1)>=_allocated)
	grow();
      
      _entries[_size].key=strdup(key);
      _entries[_size].value=strdup(value);
      _size++;
    }
  else
    {
      free(_entries[idx].value);
      _entries[idx].value=strdup(value);
    }
}

int Table::getSize()
{
  return _size;
}
