#include "main.h"
#include <io.h>
#include <fcntl.h>

HINSTANCE g_hInstance = NULL;

bool g_bIsInteractive = false;
bool g_bIsService = false;

char g_szServiceCmd[1024] = "";

// Some fun code to determine if we're running as a service
DWORD CheckSIDs(void)
{
	HANDLE hProcessToken = NULL;
	DWORD groupLength = 50;
	PTOKEN_GROUPS groupInfo = NULL;

	SID_IDENTIFIER_AUTHORITY siaNt = SECURITY_NT_AUTHORITY;
	PSID pInteractiveSid = NULL;
	PSID pServiceSid = NULL;

	DWORD dwRet = NO_ERROR;

	DWORD ndx;

	// open the token
	if (!OpenProcessToken(GetCurrentProcess(),
		TOKEN_QUERY, &hProcessToken))
	{
		dwRet = GetLastError();
		goto closedown;
	}

	// allocate a buffer of default size
	groupInfo = (PTOKEN_GROUPS)LocalAlloc(0, groupLength);
	if (groupInfo == NULL)
	{
		dwRet = GetLastError();
		goto closedown;
	}

	// try to get the info
	if (!GetTokenInformation(hProcessToken, TokenGroups,
		groupInfo, groupLength, &groupLength))
	{
		// if buffer was too small, allocate to proper size, otherwise error
		if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		{
			dwRet = GetLastError();
			goto closedown;
		}

		LocalFree(groupInfo);

		groupInfo = (PTOKEN_GROUPS)LocalAlloc(0, groupLength);
		if (groupInfo == NULL)
		{
			dwRet = GetLastError();
			goto closedown;
		}

		if (!GetTokenInformation(hProcessToken, TokenGroups,
			groupInfo, groupLength, &groupLength))
		{
			dwRet = GetLastError();
			goto closedown;
		}
	}

	//
	//  We now know the groups associated with this token.  We want
	//  to look to see if the interactive group is active in the
	//  token, and if so, we know that this is an interactive process.
	//
	//  We also look for the "service" SID, and if it's present,
	//  we know we're a service.
	//
	//  The service SID will be present iff the service is running in a
	//  user account (and was invoked by the service controller).
	//

	// create comparison sids
	if (!AllocateAndInitializeSid(&siaNt, 1, SECURITY_INTERACTIVE_RID,
		0, 0, 0, 0, 0, 0, 0, &pInteractiveSid))
	{
		dwRet = GetLastError();
		goto closedown;
	}

	if (!AllocateAndInitializeSid(&siaNt, 1, SECURITY_SERVICE_RID,
		0, 0, 0, 0, 0, 0, 0, &pServiceSid))
	{
		dwRet = GetLastError();
		goto closedown;
	}


	// reset flags
	g_bIsInteractive = false;
	g_bIsService = false;

	// try to match sids
	for (ndx = 0; ndx < groupInfo->GroupCount ; ndx += 1)
	{
		SID_AND_ATTRIBUTES sanda = groupInfo->Groups[ndx];
		PSID pSid = sanda.Sid;

		//
		//	  Check to see if the group we're looking at is one of
		//	  the two groups we're interested in.
		//

		if (EqualSid(pSid, pInteractiveSid))
		{
			//
			//  This process has the Interactive SID in its
			//  token.  This means that the process is running as
			//  a console process
			//
			g_bIsInteractive = true;
			g_bIsService = false;
			break;
		}
		else if (EqualSid(pSid, pServiceSid))
		{
			//
			//  This process has the Service SID in its
			//  token.  This means that the process is running as
			//  a service running in a user account ( not local system ).
			//
			g_bIsService = true;
			g_bIsInteractive = false;
			break;
		}
	}

	if ( !( g_bIsService || g_bIsInteractive ) )
	{
		//
		//  Neither Interactive or Service was present in the current
		//  users token, This implies that the process is running as
		//  a service, most likely running as LocalSystem.
		//
		g_bIsService = true;

		// determine if the local system service is interactive.  We
		// do this y looking in the service's registry and checking
		// the Type value for the SERVICE_INTERACTIVE_PROCESS bit

		HKEY  hkey = NULL;

		TCHAR szKey[256];

		sprintf( szKey, TEXT("SYSTEM\\CurrentControlSet\\Services\\%s"),
			SERVICE_NAME );
		if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE,
						   szKey,
						   0,
						   KEY_READ,
						   &hkey ) == ERROR_SUCCESS )
		{
			DWORD dwType = 0;
			DWORD dwSize = sizeof(DWORD);
			if ( RegQueryValueEx(hkey,
								 TEXT("Type"),
								 NULL,
								 NULL,
								 (LPBYTE) &dwType,
								 &dwSize ) == ERROR_SUCCESS )
			{
				if ( dwType & SERVICE_INTERACTIVE_PROCESS )
				{
					g_bIsInteractive = true;

					// service was given permission to interact with the
					// with the desktop, so allocate a console
					FreeConsole();
					if ( AllocConsole() )
					{
						// Now re-map the C Runtime STDIO handles
						int hCrt = _open_osfhandle((long)
							GetStdHandle(STD_OUTPUT_HANDLE), _O_TEXT);

						*stdout = *(_fdopen(hCrt, "w"));
						setvbuf(stdout, NULL, _IONBF, 0);

						*stderr = *(_fdopen(hCrt, "w"));
						setvbuf(stderr, NULL, _IONBF, 0);

					}

					DWORD dwDebug = 0;
					DWORD dwDbgSize = sizeof(dwDebug);
					TCHAR szWindbgPath[256];
					DWORD dwPathSize = sizeof(szWindbgPath);
					if (
					   ( RegQueryValueEx(hkey,
										 TEXT("Debug"),
										 NULL,
										 NULL,
										 (LPBYTE) &dwDebug,
										 &dwDbgSize ) == ERROR_SUCCESS )
					   &&
					   ( RegQueryValueEx(hkey,
										 TEXT("WindbgPath"),
										 NULL,
										 NULL,
										 (LPBYTE) szWindbgPath,
										 &dwPathSize ) == ERROR_SUCCESS )
					   )
					{
						if( dwDebug )  // 1 if debug
						{
							PROCESS_INFORMATION pi;
							STARTUPINFO		 si;		// startup info

							si.cb		  = sizeof( STARTUPINFO );
							si.lpReserved  = NULL;
							si.lpDesktop   = NULL;
							si.lpTitle	 = NULL;
							si.dwX		 = 0;
							si.dwY		 = 0;
							si.dwXSize	 = 0;
							si.dwYSize	 = 0;
							si.dwFlags	 = (DWORD) NULL;
							si.wShowWindow = SW_SHOWDEFAULT;
							si.cbReserved2 = 0;
							si.lpReserved2 = NULL;

							pi.hProcess  = NULL;

							TCHAR szCmd[300];
							_stprintf( szCmd, _T("%s -g -p 0x%x"),
								szWindbgPath, GetCurrentProcessId() );
							//-- now create the windbg process
							if( CreateProcess(
								   NULL, szCmd,
								   (LPSECURITY_ATTRIBUTES) NULL,
								   (LPSECURITY_ATTRIBUTES) NULL,
								   TRUE,
								   DETACHED_PROCESS,
								   (LPVOID) NULL,
								   (LPTSTR) NULL,
								   &si, &pi ) )
							{
								CloseHandle( pi.hThread );
								CloseHandle( pi.hProcess );
							}
						}
					}

				}
			}

			RegCloseKey( hkey );
		}


	}


	closedown:
		if ( pServiceSid )
			FreeSid( pServiceSid );

		if ( pInteractiveSid )
			FreeSid( pInteractiveSid );

		if ( groupInfo )
			LocalFree( groupInfo );

		if ( hProcessToken )
			CloseHandle( hProcessToken );

	return dwRet;
}


int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE, 
            PSTR, int) 
{
    g_hInstance = hInstance;

    CheckSIDs();

    if (g_bIsService)
    {
        SERVICE_TABLE_ENTRY ste[2] = {0};
        ste[0].lpServiceName = strdup(SERVICE_NAME);
        ste[0].lpServiceProc = ServiceMain;
        StartServiceCtrlDispatcher(ste);
    }
    else
    {
        StartWork();
    }

    return 0;
}

SERVICE_STATUS_HANDLE g_ssh;
SERVICE_STATUS g_ss;

VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR * lpszArgv)
{

    int i;
    for (i = 0; i < (signed) dwArgc; i ++)
    {
        strcat(g_szServiceCmd, lpszArgv[i]);
    }
    g_hInstance = (HINSTANCE) GetCurrentProcess();

    g_ssh = RegisterServiceCtrlHandler(SERVICE_NAME, Handler);

    g_ss.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
    g_ss.dwCurrentState = SERVICE_RUNNING;
    g_ss.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_PAUSE_CONTINUE | SERVICE_ACCEPT_SHUTDOWN;
    g_ss.dwWin32ExitCode = NO_ERROR;
    g_ss.dwCheckPoint = 0;
    g_ss.dwWaitHint = 0;

    SetServiceStatus(g_ssh, &g_ss);

    StartWork();

    g_ss.dwCurrentState = SERVICE_STOPPED;
    SetServiceStatus(g_ssh, &g_ss);

}

VOID WINAPI Handler(DWORD fdwControl)
{
    switch (fdwControl)
    {
    case SERVICE_CONTROL_STOP:
        PostMessage(g_pMainWnd->GethWnd(), WM_REALLY_CLOSE, 0, 0);
        break;
    case SERVICE_CONTROL_PAUSE:
        PostMessage(g_pMainWnd->GethWnd(), WM_USER_PAUSE, 0, 0);
        g_ss.dwCurrentState = SERVICE_PAUSED;
        SetServiceStatus(g_ssh, &g_ss);
        break;
    case SERVICE_CONTROL_CONTINUE :
        PostMessage(g_pMainWnd->GethWnd(), WM_USER_CONT, 0, 0);
        g_ss.dwCurrentState = SERVICE_RUNNING;
        SetServiceStatus(g_ssh, &g_ss);
        break;
    }
}

void StartWork()
{
    CMainWnd *  pmwMain = NULL;
    MSG         msg;

    xtrack_init();

    TiVoCrack_Init();

    InitializeCriticalSection(&g_crit);
    g_eventStop = CreateEvent(NULL, FALSE, FALSE, NULL);

    pmwMain = new CMainWnd();
    xtrack(pmwMain);

    pmwMain->Create(g_hInstance);

    while (GetMessage (&msg, NULL, 0, 0)) 
    {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }

    xuntrack(pmwMain);
    delete pmwMain;

    TiVoCrack_Uninit();

    xtrack_dump();

}