#include "xtrack.h"

#if defined(DEBUG) || defined(_DEBUG)

CRITICAL_SECTION g_critXTrack;

#include <stdio.h>
struct MEM_REF
{
	LPVOID		lpVoid;
	UINT		nLine;
    bool        bReported;
	char		szFile[MAX_PATH];
	MEM_REF*	pmrNext;
};

MEM_REF * pmrFirst = NULL;
MEM_REF * pmrLast = NULL;

void MyInit()
{
    InitializeCriticalSection(&g_critXTrack);
}

void MyReport()
{
    EnterCriticalSection(&g_critXTrack);
	MEM_REF * pmr;
	pmr = pmrFirst;
    TCHAR szOut[MAX_PATH*2];
    int nCount = 0;

    bool bFoundOne = false;

	while (pmr)
	{
        nCount ++;
        if (pmr->bReported == false)
        {
            if (bFoundOne == false)
            {
            	OutputDebugString(_T("---- Dump of all memory leaks\n"));
                bFoundOne = true;
            }

			char szIn[MAX_PATH * 2];
			sprintf(szIn, "%s(%d): Never freed!\n", pmr->szFile, pmr->nLine);
			#ifdef UNDER_CE
				DWORD dwSize = strlen(szIn);
				DWORD count = mbstowcs(NULL, szIn, dwSize);
				LPWSTR szOut = (WCHAR *) malloc ((count + 1) * sizeof(WCHAR));
				memset(szOut, 0, ((count + 1) * sizeof(WCHAR)));
				mbstowcs(szOut, szIn, dwSize);
				OutputDebugString(szOut);
				free(szOut);
			#else
				OutputDebugString(szIn);
			#endif
            pmr->bReported = true;
        }
		pmr = pmr->pmrNext;
	}

    if (bFoundOne)
    {
        _stprintf(szOut, _T("---- (Total tracked: %d)\n"), nCount);
	}
	else
	{
        _stprintf(szOut, _T("---- No memory leaks detected (Total tracked: %d)\n"), nCount);
	}
    OutputDebugString(szOut);

    LeaveCriticalSection(&g_critXTrack);
}

void* MyMalloc(UINT s, UINT l, char * f)
{
	void * lpR = malloc(s);
    MyMalloc2(lpR, l, f);
	return lpR;
}

void MyMalloc2(LPVOID s, UINT l, char * f)
{
    EnterCriticalSection(&g_critXTrack);

	MEM_REF * pmr = (MEM_REF *) malloc(sizeof(MEM_REF));

	pmr->lpVoid = s;
	pmr->nLine = l;
	pmr->pmrNext = NULL;
    pmr->bReported = false;
	strcpy(pmr->szFile, f);

	if (pmrFirst == NULL)
	{
		pmrFirst = pmr;
		pmrLast = pmr;
	}
	else
	{
		pmrLast->pmrNext = pmr;
		pmrLast = pmr;
	}
    LeaveCriticalSection(&g_critXTrack);
}

void MyFree(LPVOID lpv)
{
    MyFree2(lpv);
	free(lpv);
}

void MyFree2(LPVOID lpv)
{
    if (lpv == NULL)
    {
        return;
    }

    EnterCriticalSection(&g_critXTrack);

	bool bFound = false;
	MEM_REF * pmr = NULL;
	MEM_REF * pmrPrev = NULL;
	pmr = pmrFirst;
	while (!bFound && pmr)
	{
		if (pmr->lpVoid == lpv)
		{
			bFound = true;
		}
		else
		{
			pmrPrev = pmr;
			pmr = pmr->pmrNext;
		}
	}
	if (!bFound)
	{
		OutputDebugString(_T("Error!  Attempt to uninitalize unalloced memory!\n"));
        LeaveCriticalSection(&g_critXTrack);
		return;
	}
	if (pmrFirst == pmr)
	{
		pmrFirst = pmr->pmrNext;
	}
	if (pmrLast == pmr)
	{
		pmrLast = pmrPrev;
	}
	if (pmrPrev)
	{
		pmrPrev->pmrNext = pmr->pmrNext;
	}

    if (pmr->bReported)
    {
		char szIn[MAX_PATH * 2];
		sprintf(szIn, "%s(%d): Reported leak freed.\n", pmr->szFile, pmr->nLine);
		#ifdef UNDER_CE
			DWORD dwSize = strlen(szIn);
			DWORD count = mbstowcs(NULL, szIn, dwSize);
			LPWSTR szOut = (WCHAR *) malloc ((count + 1) * sizeof(WCHAR));
			memset(szOut, 0, ((count + 1) * sizeof(WCHAR)));
			mbstowcs(szOut, szIn, dwSize);
			OutputDebugString(szOut);
			free(szOut);
		#else
			OutputDebugString(szIn);
		#endif
    }

	free(pmr);
    LeaveCriticalSection(&g_critXTrack);
}

char* MyStrDup(char * szIn, UINT l, char * f)
{
    char * szTemp = strdup(szIn);
    MyMalloc2(szTemp, l, f);
    return szTemp;
}

#endif //DEBUG
